'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _isInteger = require('babel-runtime/core-js/number/is-integer');

var _isInteger2 = _interopRequireDefault(_isInteger);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _yamljs = require('yamljs');

var _yamljs2 = _interopRequireDefault(_yamljs);

var _dispatcher = require('./lib/dispatcher');

var _dispatcher2 = _interopRequireDefault(_dispatcher);

var _datadir = require('./lib/validators/datadir');

var _datadir2 = _interopRequireDefault(_datadir);

var _log = require('./lib/log');

var _log2 = _interopRequireDefault(_log);

var _gui = require('./lib/servers/gui');

var _gui2 = _interopRequireDefault(_gui);

var _ota = require('./lib/servers/ota');

var _ota2 = _interopRequireDefault(_ota);

var _mqtt = require('./lib/clients/mqtt');

var _mqtt2 = _interopRequireDefault(_mqtt);

var _infrastructure = require('./lib/infrastructure');

var _infrastructure2 = _interopRequireDefault(_infrastructure);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_UI_PORT = 80;
var DEFAULT_DATADIR = _path2.default.join(_os2.default.homedir(), '/.homie');

var config = {};

var bootstrap = function bootstrap(options) {
  var uiPort = options.uiPort || DEFAULT_UI_PORT;
  var dataDir = options.dataDir || DEFAULT_DATADIR;
  if (typeof options.logLevel !== 'undefined') {
    _log2.default.setLogLevel(options.logLevel);
  }

  _log2.default.info('Using data directory ' + dataDir);

  var fail = function fail(message) {
    _log2.default.fatal(message);
    process.exit(1);
  };

  if (!(0, _isInteger2.default)(uiPort) || uiPort < 1 || uiPort > 65535) {
    fail('UI port must be a valid port');
  }

  var mkdirIfNotExisting = function mkdirIfNotExisting(dir) {
    try {
      _fs2.default.mkdirSync(dir);
    } catch (err) {
      return;
    }
  };

  var mkyamlIfNotExisting = function mkyamlIfNotExisting(path, object) {
    try {
      _fs2.default.accessSync(path, _fs2.default.R_OK | _fs2.default.W_OK);
    } catch (err) {
      _fs2.default.writeFileSync(path, _yamljs2.default.stringify(object, null, 2), 'utf8');
    }
  };

  mkdirIfNotExisting(dataDir);
  mkyamlIfNotExisting(_path2.default.join(dataDir, '/infrastructure.yml'), { devices: [], groups: [] });
  mkyamlIfNotExisting(_path2.default.join(dataDir, '/config.yml'), {});
  mkdirIfNotExisting(_path2.default.join(dataDir, '/ota'));
  mkyamlIfNotExisting(_path2.default.join(dataDir, '/ota/manifest.yml'), { firmwares: [] });
  mkdirIfNotExisting(_path2.default.join(dataDir, '/ota/firmwares'));
  mkdirIfNotExisting(_path2.default.join(dataDir, '/db'));

  var infrastructure = _yamljs2.default.load(_path2.default.join(dataDir, '/infrastructure.yml'));
  if (!_datadir2.default.validateInfrastructure(infrastructure)) {
    fail('infrastructure.yml is invalid');
  }

  var configFile = _yamljs2.default.load(_path2.default.join(dataDir, '/config.yml'));
  if (!_datadir2.default.validateConfig(configFile)) {
    fail('config.yml is invalid');
  }

  config.dataDir = dataDir;
  config.uiPort = uiPort;

  config.file = configFile;

  start();
};

var start = function start() {
  var servers = [{
    Class: _gui2.default,
    params: { port: config.uiPort }
  }, {
    Class: _ota2.default,
    params: { dataDir: config.dataDir }
  }];

  var entitiesReadyCount = 0;
  var entityReady = function entityReady() {
    if (++entitiesReadyCount === servers.length + 1 + 1) {
      // +1 for infrastructure and mqtt
      _log2.default.info('Homie is ready');
      _dispatcher2.default.start();
    }
  };

  var infrastructure = new _infrastructure2.default({ dataDir: config.dataDir });
  infrastructure.on('ready', function () {
    _log2.default.info('Infrastructure loaded');
    _dispatcher2.default.attach('infrastructure', infrastructure);
    entityReady();
  });
  infrastructure.start();

  var mqttClient = new _mqtt2.default(config.file.mqtt);
  mqttClient.on('ready', function () {
    _log2.default.info('MQTT client connected');
    _dispatcher2.default.attach('mqtt', mqttClient);
    entityReady();
  });
  mqttClient.start();

  servers.forEach(function (server) {
    var serverInstance = new server.Class(server.params);

    serverInstance.on('ready', function (data) {
      _log2.default.info(serverInstance.getName() + ' server listening on ' + data.host + ':' + data.port);
      _dispatcher2.default.attach(serverInstance.getName().toLowerCase(), serverInstance);
      entityReady();
    });

    serverInstance.on('error', function (err) {
      _log2.default.fatal(serverInstance.getName() + ' server cannot listen', err);
      process.exit(1);
    });

    serverInstance.start();
  });
};

exports.default = bootstrap;